package com.chelvc.cloud.vehicle.server.service.impl;

import java.math.BigDecimal;
import java.time.LocalDateTime;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.server.dao.AssetMapper;
import com.chelvc.cloud.vehicle.server.entity.Asset;
import com.chelvc.cloud.vehicle.server.service.AssetService;
import com.chelvc.framework.common.exception.ResourceUnavailableException;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class AssetServiceImpl extends ServiceImpl<AssetMapper, Asset> implements AssetService {
    @Override
    public Asset queryAsset(Long userId, Integer type) {
        LambdaQueryWrapper<Asset> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(Asset::getUserId, userId);
        wrapper.eq(Asset::getType, type);
        Asset asset = baseMapper.selectOne(wrapper);
        if (asset != null) {
            return asset;
        }
        asset = new Asset();
        asset.setFrozen(new BigDecimal("0"));
        asset.setType(type);
        asset.setTotal(new BigDecimal("0"));
        asset.setUpdateTime(LocalDateTime.now());
        asset.setUserId(userId);
        baseMapper.insert(asset);
        return asset;
    }

    @Override
    public BigDecimal updateAsset(Integer operateType, BigDecimal operateAmount, Long userId, Integer type) {
        Asset asset = queryAsset(userId, type);
        BigDecimal total = asset.getTotal();
        if (operateType == 1 && total.compareTo(operateAmount) < 0) {
            throw new ResourceUnavailableException("账户余额不足");
        }
        if (operateType == 0) {
            asset.setTotal(total.add(operateAmount));
        } else if (operateType == 1) {
            asset.setTotal(total.subtract(operateAmount));
        }
        int i = baseMapper.updateById(asset);
        if (i != 1) {
            throw new ResourceUnavailableException("操作失败");
        }
        return asset.getTotal();
    }
}
