package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.api.dto.BalanceDetailDTO;
import com.chelvc.cloud.vehicle.api.param.QueryMerchantBalanceParam;
import com.chelvc.cloud.vehicle.server.dao.BalanceDetailMapper;
import com.chelvc.cloud.vehicle.server.entity.BalanceDetail;
import com.chelvc.cloud.vehicle.server.service.BalanceDetailService;
import com.chelvc.framework.common.model.Pagination;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDateTime;

@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class BalanceDetailServiceImpl extends ServiceImpl<BalanceDetailMapper, BalanceDetail>
        implements BalanceDetailService, com.chelvc.cloud.vehicle.api.service.BalanceDetailService{

    private final BalanceDetailMapper merchantBalanceDetailMapper;
    @Override
    public void recordFlow(Long orderId, Integer type, Long userId, BigDecimal operateAmount, BigDecimal surplusAmount,
                           Integer flowType, Integer operateType, Long sourceId) {
        BalanceDetail detail = new BalanceDetail();
        detail.setType(type);
        detail.setFlowType(flowType);
        detail.setOperateType(operateType);
        detail.setOrderId(orderId);
        detail.setUserId(userId);
        detail.setCreateTime(LocalDateTime.now());
        detail.setOperateAmount(operateAmount);
        detail.setSurplusAmount(surplusAmount);
        detail.setSourceId(sourceId);
        baseMapper.insert(detail);
    }

    @Override
    public Pagination<BalanceDetailDTO> queryByUserId(QueryMerchantBalanceParam param,
                    Integer pageNum, Integer pageSize, Long userId, Integer type) {
        if(param == null || userId == null){
            return null;
        }
        Page<BalanceDetailDTO> page = new Page<>(pageNum, pageSize);
        IPage<BalanceDetailDTO> result = merchantBalanceDetailMapper
                .queryByUserId(page, queryWrapper(param, userId, type));
        return Pagination.<BalanceDetailDTO>builder().total(result.getTotal()).pages(result.getPages())
                .records(result.getRecords()).build();
    }

    public QueryWrapper<BalanceDetail> queryWrapper(QueryMerchantBalanceParam param, Long userId, Integer type) {
        QueryWrapper<BalanceDetail> lqw = Wrappers.query();
        lqw.eq("t1.type", type);
        lqw.eq("t1.user_id", userId);
        if(param.getType() != null){
            lqw.eq("t1.type", param.getType());
        }
        if(param.getOperateType() != null){
            lqw.eq("t1.operate_type", param.getOperateType());
        }
        if(param.getStartTime() != null){
            lqw.ge("t1.create_time", param.getStartTime());
        }
        if(param.getEndTime() != null){
            lqw.lt("t1.create_time", param.getEndTime());
        }
        lqw.orderByDesc("t1.create_time");
        return lqw;
    }
}
