package com.chelvc.cloud.vehicle.server.service.impl;

import java.math.BigDecimal;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.uc.api.dto.UserDTO;
import com.chelvc.cloud.uc.api.model.Scope;
import com.chelvc.cloud.uc.api.service.EmployeeService;
import com.chelvc.cloud.uc.api.service.UserService;
import com.chelvc.cloud.vehicle.api.constant.MerchantStatus;
import com.chelvc.cloud.vehicle.api.dto.CategoryDTO;
import com.chelvc.cloud.vehicle.api.dto.GoodsDTO;
import com.chelvc.cloud.vehicle.api.dto.MerchantDTO;
import com.chelvc.cloud.vehicle.api.dto.MerchantDetailDTO;
import com.chelvc.cloud.vehicle.api.dto.MerchantRankCountDTO;
import com.chelvc.cloud.vehicle.api.dto.MerchantRankDTO;
import com.chelvc.cloud.vehicle.api.param.GoodsQueryParam;
import com.chelvc.cloud.vehicle.api.param.LocationQueryParam;
import com.chelvc.cloud.vehicle.api.param.MerchantModifyParam;
import com.chelvc.cloud.vehicle.api.param.MerchantPagingParam;
import com.chelvc.cloud.vehicle.api.param.MerchantQueryParam;
import com.chelvc.cloud.vehicle.api.param.MerchantRankParam;
import com.chelvc.cloud.vehicle.api.param.OmsCartItemPagingParam;
import com.chelvc.cloud.vehicle.api.util.DateTimeUtils;
import com.chelvc.cloud.vehicle.server.copier.MerchantCopier;
import com.chelvc.cloud.vehicle.server.dao.MerchantMapper;
import com.chelvc.cloud.vehicle.server.dao.RegionMapper;
import com.chelvc.cloud.vehicle.server.entity.Merchant;
import com.chelvc.cloud.vehicle.server.entity.Region;
import com.chelvc.cloud.vehicle.server.service.CategoryService;
import com.chelvc.cloud.vehicle.server.service.GoodsService;
import com.chelvc.cloud.vehicle.server.service.MerchantService;
import com.chelvc.cloud.vehicle.server.service.OmsCartItemService;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import com.chelvc.framework.location.Address;
import com.chelvc.framework.location.LocationHandler;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.apache.dubbo.config.annotation.DubboReference;
import org.apache.dubbo.config.annotation.DubboService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.scheduling.annotation.Scheduled;

/**
 * 商家业务操作实现
 *
 * @author Woody
 * @date 2023/5/2
 */
@RequiredArgsConstructor(onConstructor = @__({@Autowired, @Lazy}))
@DubboService(interfaceClass = com.chelvc.cloud.vehicle.api.service.MerchantService.class)
public class MerchantServiceImpl extends ServiceImpl<MerchantMapper, Merchant> implements MerchantService,
        com.chelvc.cloud.vehicle.api.service.MerchantService {
    private final GoodsService goodsService;

    private final CategoryService categoryService;

    private final LocationHandler locationHandler;

    private final RegionMapper regionMapper;

    private final OmsCartItemService omsCartItemService;

    @DubboReference
    EmployeeService employeeService;

    @DubboReference
    UserService userService;

    @Override
    public List<MerchantDTO> listNearbyMerchants(@NonNull LocationQueryParam param, int size) {
        return this.baseMapper.listNearbyMerchants(param, size);
    }

    @Override
    public List<MerchantDTO> listRecommendMerchants(@NonNull LocationQueryParam param, int size) {
        return this.baseMapper.listRecommendMerchants(param, size);
    }

    @Override
    public List<MerchantDTO> listSimpleMerchants(@NonNull MerchantQueryParam param) {
        return this.baseMapper.listSimpleMerchants(param);
    }

    @Override
    public MerchantDetailDTO getMerchantDetail(@NonNull Long id) {
        // 获取商家信息
        MerchantDTO merchant = MerchantCopier.INSTANCE.copying(
                AssertUtils.available(this.getById(id), "商家不存在")
        );

        // 获取商家全部商品列表
        List<GoodsDTO> goods = this.goodsService.listMerchantSimpleGoods(
                id, GoodsQueryParam.builder().size(10).build()
        );

        // 获取商家商品分类类型列表
//        List<CategoryType> types = this.goodsService.listMerchantGoodsCategoryTypes(id);
        List<Long> idSet = goods.stream()
                .map(GoodsDTO::getCategoryId) // 将每个Person对象映射到其id
                .collect(Collectors.toList());
        List<CategoryDTO> categories = this.categoryService.listCategories(idSet);
        return MerchantDetailDTO.builder().merchant(merchant).goods(goods).categories(categories).build();
    }

    @Override
    public MerchantDetailDTO getMerchantDetail(Long id, LocationQueryParam param) {
        // 获取商家信息
        MerchantDTO merchant = MerchantCopier.INSTANCE.copying(
                AssertUtils.available(this.baseMapper.getMerchantById(id, param), "商家不存在")
        );
        Long userId = SessionContextHolder.getSession().getId();
        //查询商家是否被收藏
        Integer count = this.baseMapper.merchantFavorite(id, userId);
        if (count > 0) {
            merchant.setFavorites(true);
        }

        // 获取商家全部商品列表
        List<GoodsDTO> goods = this.goodsService.listMerchantSimpleGoods(
                id, GoodsQueryParam.builder().size(10).build()
        );

        // 获取商家商品分类类型列表
//        List<CategoryType> types = this.goodsService.listMerchantGoodsCategoryTypes(id);
        List<Long> idSet = goods.stream()
                .map(GoodsDTO::getCategoryId) // 将每个Person对象映射到其id
                .collect(Collectors.toList());
        if (idSet.isEmpty()) {
            return MerchantDetailDTO.builder().merchant(merchant).goods(goods).build();
        }
        List<CategoryDTO> categories = this.categoryService.listCategories(idSet);
        return MerchantDetailDTO.builder().merchant(merchant).goods(goods).categories(categories).build();
    }

    @Override
    public Long addMerchant(@NonNull MerchantModifyParam param) {
        // 判断商家名称是否存在
        Integer count = this.lambdaQuery()
                .eq(Merchant::getName, param.getName())
                .eq(Merchant::getStatus, MerchantStatus.ONLINE).count();
        AssertUtils.available(count < 1, "商家名称已存在");
        Merchant merchant = MerchantCopier.INSTANCE.copying(param);
        merchant.setUserId(0L);
        merchant.setScore(0D);
        merchant.setSale(0);
        merchant.setStatus(MerchantStatus.ONLINE);
        this.save(merchant);
        return merchant.getId();
    }

    @Override
    public void updateMerchant(@NonNull Long id, @NonNull MerchantModifyParam param) {
        Merchant merchant = AssertUtils.available(this.getById(id), "商家不存在");
        MerchantCopier.INSTANCE.copying(param, merchant);
        this.updateById(merchant);
    }

    @Override
    public void deleteMerchant(@NonNull Long id) {
        AssertUtils.available(this.getById(id), "商家不存在");
        this.baseMapper.deleteById(id);
    }

    @Override
    public Pagination<MerchantDTO> getMerchantPaging(@NonNull MerchantPagingParam param) {
        Long userId = SessionContextHolder.getId();
        Scope scope = com.chelvc.framework.common.util.StringUtils.ifEmpty(SessionContextHolder.getScope(), Scope::parse);
        if (scope == Scope.ADMIN) {
        }else {
            List<Long> merchants = this.getMerchantIdsByUserId(userId);
            if (ObjectUtils.isEmpty(merchants)) {
                return null;
            }
            param.setMerchantId(merchants.get(0));
        }
        Page<Merchant> page = this.lambdaQuery()
                .like(StringUtils.notEmpty(param.getName()), Merchant::getName, param.getName())
                .eq(Objects.nonNull(param.getRecommend()), Merchant::getRecommend, param.getRecommend())
                .eq(StringUtils.notEmpty(param.getMerchantId()),Merchant::getId,param.getMerchantId())
                .eq(Objects.nonNull(param.getStatus()), Merchant::getStatus, param.getStatus())
                .orderByDesc(Merchant::getCreateTime).page(DatabaseContextHolder.page(param.getPaging()));
        return DatabaseContextHolder.pagination(page, MerchantCopier.INSTANCE::copying);
    }

    @Override
    public MerchantDTO getMerchant(@NonNull Long id) {
        Merchant merchant = AssertUtils.available(this.getById(id), "商家不存在");
        return this.convert(merchant);
    }

    /**
     * 转换商家信息
     *
     * @param merchant 商家实例
     * @return 商家信息
     */
    private MerchantDTO convert(Merchant merchant) {
        return MerchantCopier.INSTANCE.copying(merchant);
    }


    @Override
    public List<Long> getMerchantIdsByUserId(Long userId) {
        List<Merchant> merchants = this.lambdaQuery()
                .eq(Merchant::getUserId, userId)
                .eq(Merchant::getStatus, MerchantStatus.ONLINE)
                .list();
        return merchants.stream().map(Merchant::getId).collect(Collectors.toList());
    }

    @Override
    public List<MerchantDTO> getMerchantIdsByUserId() {
        Long userId = SessionContextHolder.getSession().getId();
        List<Merchant> merchants = this.lambdaQuery()
                .eq(Merchant::getUserId, userId)
                .eq(Merchant::getStatus, MerchantStatus.ONLINE)
                .list();
        return MerchantCopier.INSTANCE.copying(merchants);
    }

    @Override
    public List<MerchantRankDTO> listRankMerchants(MerchantRankParam param, Long userId) {
        List<MerchantRankDTO> result = new ArrayList<>();
        Integer regionId = param.getRegionId();
        Double longitude = (double) 0;
        Double latitude = (double) 0;
        if (regionId == null) {
            UserDTO user = userService.getUser(userId);
            if (user == null) {
                return result;
            }
            Address address = locationHandler.ip2address(SessionContextHolder.getHost());
            if (address == null) {
                return result;
            }
            regionId = address.getDistrict().getId();
            longitude = address.getLongitude();
            latitude = address.getLatitude();
        }
        Region region = regionMapper.selectById(regionId);
        if (region == null) {
            return result;
        }
        result = baseMapper.listRankMerchants(DateTimeUtils.getMonthStartLocalTime(), regionId, longitude, latitude,
                region.getLevel());
        return result;
    }

    @Override
    public List<MerchantRankCountDTO> rankMerchants(Integer regionId, Integer limit, Integer level) {
        return baseMapper.rankMerchants(DateTimeUtils.getLastMonthStartLocalTime(),
                DateTimeUtils.getLastMonthEndLocalTime(), regionId, limit, level);
    }

    @Override
    public BigDecimal queryTotalCommissionAmountByLastMonth(Integer regionId, Integer level) {
        return baseMapper.queryTotalCommissionAmountByLastMonth(DateTimeUtils.getLastMonthStartLocalTime(),
                DateTimeUtils.getLastMonthEndLocalTime(), regionId, level);
    }

    @Override
    public MerchantDTO getMerchantById(Long merchantId) {
        return MerchantCopier.INSTANCE.copying(this.getById(merchantId));
    }

    @Override
    public String getMerchantName(Long merchantId) {
        LambdaQueryWrapper<Merchant> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(Merchant::getUserId, merchantId);
        Merchant merchant = baseMapper.selectOne(wrapper);
        return merchant != null ? merchant.getName() : "";
    }

    /**
     * 购物车列表
     *
     * @param param
     * @return
     */
    @Override
    public Pagination<MerchantDTO> getUserMineCartPaging(OmsCartItemPagingParam param) {
        Long userId = SessionContextHolder.getSession().getId();
        List<Long> merchants = this.baseMapper.getMerchantIds(userId);
        if (merchants == null && merchants.size() > 0) {
            return null;
        }
        Page<Merchant> page = this.lambdaQuery()
                .like(StringUtils.notEmpty(param.getMerchantName()), Merchant::getName, param.getMerchantName())
                .eq(Merchant::getStatus, MerchantStatus.ONLINE)
                .in(Merchant::getId, merchants)
                .orderByDesc(Merchant::getCreateTime).page(DatabaseContextHolder.page(param.getPaging()));
        Pagination<MerchantDTO> pagination = DatabaseContextHolder.pagination(page, MerchantCopier.INSTANCE::copying);
        pagination.getRecords().forEach(item -> {
            item.setCartItems(this.omsCartItemService.merchantListCartItem(item.getId()));
        });
        return pagination;
    }

    @Override
    public void updateMerchantBusinessStatus(Long merchantId, String businessStatus) {
        Merchant merchant = AssertUtils.nonnull(this.getById(merchantId), "商家不存在");
        this.baseMapper.updateMerchantBusinessStatus(merchantId, businessStatus, "1");
    }

    /**
     * 每天定时更新商家得营业状态
     */
    @Scheduled(cron = "5 * * * * ?")
    public void batchMerchantBusinessStatus() {
        List<Merchant> merchantList = this.lambdaQuery().list();
        merchantList.forEach(item -> {
            String opening = "10:00:00,18:00:00";
            String[] openStr = opening.split(",");
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("HH:mm:ss");
            LocalTime time1 = LocalTime.parse(openStr[0], formatter);
            LocalTime time2 = LocalTime.parse(openStr[1], formatter);
            if (time1.isAfter(LocalTime.now()) && time2.isAfter(LocalTime.now())) {
                this.baseMapper.updateMerchantBusinessStatus(item.getId(), "0", "0");
            } else {
                this.baseMapper.updateMerchantBusinessStatus(item.getId(), "1", "0");
            }
        });

    }


    @Override
    public MerchantDTO getMerchantByUserId(Long userId){
        Merchant merchants = this.lambdaQuery()
                .eq(Merchant::getUserId, userId)
                .eq(Merchant::getStatus, MerchantStatus.ONLINE)
                .one();
        return MerchantCopier.INSTANCE.copying(merchants);
    }



    /**
     * 获取商家营业状况
     * @return
     */
    @Override
    public Map<String,Object> operatConditions(){
        Long merchantId = null;
        Map<String,Object> map = new HashMap<>();
        Long userId = SessionContextHolder.getId();
        Scope scope = com.chelvc.framework.common.util.StringUtils.ifEmpty(SessionContextHolder.getScope(), Scope::parse);
        if (scope == Scope.ADMIN) {
        }else {
            List<Long> merchants = this.getMerchantIdsByUserId(userId);
            if (ObjectUtils.isEmpty(merchants)) {
                return null;
            }
            merchantId = merchants.get(0);
        }
        map = this.baseMapper.operatConditions(merchantId);

        return map;
    }


    /**
     * 经营报表
     * @return 经营报表信息
     */
    @Override
    public List<Map<String,Object>> operatReport(){
        Long merchantId = null;
        List<Map<String,Object>> mapList = new ArrayList<>();
        Long userId = SessionContextHolder.getId();
        Scope scope = com.chelvc.framework.common.util.StringUtils.ifEmpty(SessionContextHolder.getScope(), Scope::parse);
        if (scope == Scope.ADMIN) {
        }else {
            List<Long> merchants = this.getMerchantIdsByUserId(userId);
            if (ObjectUtils.isEmpty(merchants)) {
                return null;
            }
            merchantId = merchants.get(0);
        }
        mapList = this.baseMapper.operatReport(merchantId);

        return mapList;
    }
}
